#include "LED_SEG.h"

//Each 8-bit segment code is encoded in the following order: Dot-SegG-SegF-SegE-SegD-SegC-SegB-SegA. 
static unsigned char NumSegCode[10] = {0xC0, 0xF9, 0xA4, 0xB0, 0x99, 0x92, 0x82, 0xF8, 0x80, 0x90};	// 0~9
static unsigned char FreqSegCodeArray[4] = {0xFF, 0xFF, 0xFF, 0xFF};					//Shut down all LED segments.

/*88.:8.8.

                   --A--       --A--        --A--       --A--
                  F     B     F     B      F     B     F     B
                  |     |     |     |  D3  |     |     |     |
                   --G--       --G--        --G--       --G-- 
                  E     C     E     C  D4  E     C     E     C
                  |     |     |     |      |     |     |     |
                   --D--  D1   --D--  D2    --D--  D5   --D--  D6

                    Digit0----->Digit1------>Digit2----->Digit3
        _______________|           |            |           |____________
       |                           |            |                        |
FreqSegCodeArray[0]->FreqSegCodeArray[1]->FreqSegCodeArray[2]->FreqSegCodeArray[3]

Scan order: from left to right.  	

There is no internal connection for "D1".
"D3" and "D4" are tied together internally.
*/

void SegDisplayInit(void)
{
	Digit0Select = 0;
	Digit1Select = 0;
	Digit2Select = 0;
	Digit3Select = 0;						//Shut down all four digits.					

	TMOD &= 0xF0;
	TMOD |= 0x01;								//Timer 0 works in Mode 1 (16-bit Timer).

	TH0 = TH0_VALUE;
	TL0 = TL0_VALUE;				
	
	ET0 = 1;										//Timer 0 Interrupt Enable
	TR0 = 1;										//Enable Timer 0.
}

void SegCodeToIO(unsigned char SegCode)
{
	SegDot = SegCode >> 7;
	SegG = (SegCode >> 6) & 0x01;
	SegF = (SegCode >> 5) & 0x01;
	SegE = (SegCode >> 4) & 0x01;
	SegD = (SegCode >> 3) & 0x01;
	SegC = (SegCode >> 2) & 0x01;
	SegB = (SegCode >> 1) & 0x01;
	SegA = SegCode & 0x01;
}

void SegDisplayFreq(unsigned short Freq)				//The parameter "Freq" is the actual Frequency multiplied by 10.
{
	if(Freq >= 1000)
	{
		FreqSegCodeArray[0] = 0xF9;									//"1"
		FreqSegCodeArray[1] = 0xC0;									//"0"
	}
	else
	{
		FreqSegCodeArray[0] = 0xFF;									//Blank
		FreqSegCodeArray[1] = NumSegCode[Freq/100];
	}
	FreqSegCodeArray[2] = NumSegCode[(Freq%100)/10];
	FreqSegCodeArray[2] &= 0x7F;									//Add the decimal point.
	FreqSegCodeArray[3] = NumSegCode[Freq%10];
}

void Timer0ISR(void) interrupt 1 using 2
{
	static unsigned char DigitSel = 0;						//This is the counter for rolling loop scan. 

	TH0 = TH0_VALUE;
	TL0 = TL0_VALUE;															//Reload Timer 0. Timer 0 interrupt flag is cleared by hardware automatically.

	switch(DigitSel)
	{
		case 0: 
		{
			Digit3Select = 0;
			SegCodeToIO(FreqSegCodeArray[0]);
			Digit0Select = 1;
			DigitSel++;
			break;
		}
		case 1:
		{
			Digit0Select = 0;
			SegCodeToIO(FreqSegCodeArray[1]);
			Digit1Select = 1;
			DigitSel++;
			break;
		}
		case 2:
		{
			Digit1Select = 0;
			SegCodeToIO(FreqSegCodeArray[2]);
			Digit2Select = 1;
			DigitSel++;
			break;
		}
		case 3:
		{
			Digit2Select = 0;
			SegCodeToIO(FreqSegCodeArray[3]);
			Digit3Select = 1;
			DigitSel = 0;
			break;
		}
	}	
}